package application.app.view
{
	import flash.geom.Point;
	
	import flash.display.Sprite;

	import application.app.ApplicationFacade;

	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;

	/**
	 * Permet de construire notre menu et d'interagir avec nos composants graphiques.
	 * 
	 * @author matth
	 */
	public class ViewKartIA extends Mediator implements IMediator
	{
		public static const NAME : String = "ViewKartIA";// intelligence artificielle
		
		private var oKart : Sprite;
		public var oLineTestRace : Sprite;// ligne de test du circuit	
		private var oRace : Sprite;// référence au clip circuit
				
		/* variables ajustement kart */
		public var speed_decay:Number;
		public var rotation_step:Number;// vitesse de rotation
	
		public var nAcceleration : Number;// accélération du kart
		public var nMaxSpeed : Number;// vitesse maximum du kart
		public var nSpeed : Number;// vitesse du kart
		
		
		public var nAngleTest : Number;// angle pour la ligne de test		
		public var nPrecision : int;// la précision: plus c'est élevée plus c'est précis, et plus lent pour la simulation		
		public var nSizeTest : int;// la longueur pour les tests d'anticipation des collisions	
		public var nNumberSteps : int;// le nombre d'étape requis pour valider les tests		
		public var nTurnTolerance : int;// la tolérance pour tourner

		
		
		public function ViewKartIA(viewComponent : Object)
		{
			super(NAME, viewComponent);
			this.setupUi();
		}


		/**
		 * Initialisation de l'interface utilisateur.
		 * 
		 */
		private function setupUi() : void
		{
			// création et configuration du kart
			oKart = new KartIA1();
			stageV.addChild(oKart);
			
			oLineTestRace = new Sprite();
			oKart.addChild(oLineTestRace);
		}


		/**
		 * Initialisation des variables pour une nouvelle partie
		 * 
		 */
		private function startGame() : void
		{
			// position de départ du kart
			oKart.x = 450;
			oKart.y = 300;
			oKart.rotation = 0;
			
			nAngleTest = 90;
			nPrecision = 13;
			nSizeTest = 63;
			nNumberSteps = 13;
			nTurnTolerance = 13;
			
			speed_decay = 0.96;
			rotation_step = 10;// vitesse de rotation
			nAcceleration = 0.33;// accélération du kart
			nMaxSpeed = 5;// vitesse maximum du kart
			nSpeed = 0;// vitesse du kart
		}

			
		/**
		 * Déplace le kart IA en fonction des tests de collision du circuit.
		 * 
		 */
		private function moveKart() : void
		{
			var angle_step : Number = nAngleTest * 2 / nPrecision;
			var sight_point : Point;
			var global_sight_point : Point;
			
			var turn_left : int = 0;// contient le résultat du test de collision
			var bCollision : Boolean = false;// collision avec le circuit ?
			
			oLineTestRace.graphics.clear();
			oLineTestRace.graphics.lineStyle(1, 0x00C5DF);
			
			// détermine la prochaine trajectoire du kart (test avec les lignes)
			for (var i : int = 0; i <= nPrecision; i++)
			{
				for (var j : int = 1; j <= nNumberSteps; j++)
				{
					oLineTestRace.graphics.moveTo(0, -15);
					sight_point = new Point(nSizeTest / nNumberSteps * j * Math.cos((-90 - nAngleTest + angle_step * i) * 0.0174532925), nSizeTest / nNumberSteps * j * Math.sin((-90 - nAngleTest + angle_step * i) * 0.0174532925));
					global_sight_point = oKart.localToGlobal(sight_point);
					
					// quitte la boucle en cas de collision prévue du circuit et du kart
					if (oRace.hitTestPoint(global_sight_point.x, global_sight_point.y, true))	
						break;
				}
				
				if (i < nPrecision / 2)	turn_left += j;
				else	turn_left -= j;

				oLineTestRace.graphics.lineTo(sight_point.x, sight_point.y);
			}
			
			// accélération si turn_left est plus petit que la tolerance * 2
			if (nSpeed < nMaxSpeed && Math.abs(turn_left) < nTurnTolerance * 2)
				nSpeed += nAcceleration;
			
			var speed_x : Number = Math.sin(oKart.rotation * 0.0174532925) * nSpeed;
			var speed_y : Number = - Math.cos(oKart.rotation * 0.0174532925) * nSpeed;
			
			oKart.y += speed_y;// mise à jour de la position du kart IA
			oKart.x += speed_x;
			
			var point_left : Point = new Point(-9, 0);
			var point_right : Point = new Point(9, 0);
			var point_front : Point = new Point(0, -13);
			var point_back : Point = new Point(0, 13);
			
			point_left = oKart.localToGlobal(point_left);
			point_right = oKart.localToGlobal(point_right);
			point_front = oKart.localToGlobal(point_front);
			point_back = oKart.localToGlobal(point_back);
			
			// gestion voiture et circuit collision
			if (oRace.hitTestPoint(point_left.x, point_left.y, true) && oRace.hitTestPoint(point_front.x, point_front.y, true))
			{
				oKart.rotation += 5;
				nSpeed *= 0.85;
				bCollision = true;
			}
			
			if (oRace.hitTestPoint(point_right.x, point_right.y, true) && oRace.hitTestPoint(point_front.x, point_front.y, true))
			{
				oKart.rotation -= 5;
				nSpeed *= 0.85;
				bCollision = true;
			}
			
			if (oRace.hitTestPoint(point_front.x, point_front.y, true))
			{
				nSpeed *= 0.55;
				bCollision = true;
			}
			
			if (oRace.hitTestPoint(point_back.x, point_back.y, true))
			{
				nSpeed *= 0.55;
				bCollision = true;
			}
			
			if (!bCollision)
			{
				// tourne à gauche ou à droite selon la collision du kart et des limites du circuit
				if (Math.abs(turn_left) > nTurnTolerance)
				{
					if (turn_left > 0)	oKart.rotation -= rotation_step * (nSpeed / nMaxSpeed);
					else	oKart.rotation += rotation_step * (nSpeed / nMaxSpeed);
				}
			}
			
			if (Math.abs(nSpeed) > 0.3)	nSpeed *= speed_decay;
			else	nSpeed = 0;					
		}

		
		/**
		 * Transtypage du viewComponent en son type actuel.
		 * 
		 */
		protected function get stageV() : Sprite
		{
			return viewComponent as Sprite;
		}

		/**
		 * Notifications attendues.
		 * 
		 */
		override public function listNotificationInterests() : Array
		{
			return [ApplicationFacade.ON_ENTER_FRAME,
					ApplicationFacade.RACE_READY];
		}

		/**
		 * Gestion des notifications.
		 * 
		 */
		override public function handleNotification(note : INotification) : void
		{
			switch (note.getName())
			{
				case ApplicationFacade.RACE_READY:
					oRace = Sprite( note.getBody() );
					this.startGame();
					break;
												
				case ApplicationFacade.ON_ENTER_FRAME:
					if( oRace != null )	this.moveKart();
					break;	
			}
		}

	}
}
